@@ -16,7 +16,7 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# Copyright 2000-2004 Free Software Foundation
+# Copyright 2000-2005 Free Software Foundation
 #
 # FILE:
 # GBaseApp.py
@@ -34,11 +34,13 @@ import ConfigParser
 
 from gnue import paths
 
-from gnue.common.apps import GConfig
-from gnue.common.apps import GDebug
+from gnue.common.apps import errors, GConfig, GDebug
 from gnue.common.datasources import GConnections
 from CommandOption import CommandOption
 
+class StartupError (errors.UserError):
+  pass
+
 class GBaseApp:
   """
   The base class of the various GNUe application classes.
@@ -79,107 +81,118 @@ class GBaseApp:
   ARGUMENTS = []      # Will contain an array of command line arguments
   connections = None  # Will contain a GConnection object
 
-  def __init__(self, connections=None, application=None, defaults=None):
+  def __init__(self, connections=None, application='common', defaults=None):
+
+    self.configDefaults = defaults 
+    
+    sys.excepthook = self.excepthook    
 
     # Basic options
     self._base_options = [
+      ##
+      ## Base options
+      ##
+      CommandOption ('version', category = "base", action = self.doVersion,
+        help = _('Displays the version information for this program.')),
 
-         ##
-         ## Base options
-         ##
+      CommandOption ('debug-level', category = "base", default = 0,
+        argument = _("level"),
+        help = _('Enables debugging messages.  Argument specifies the '
+                 'level of messages to display (e.g., "--debug-level 5" '
+                 'displays all debugging messages at level 5 or below.)')),
 
-         CommandOption('version', category="base", action=self.doVersion,
-              help=_('Displays the version information for this program.') ),
+      CommandOption ('debug-file', category = "base", argument = _("filename"),
+        help = _('Sends all debugging messages to a specified file '
+                 '(e.g., "--debug-file trace.log" sends all output to '
+                 '"trace.log")')),
 
-         CommandOption('debug-level', category="base", default=0, argument=_("level"),
-              help=_('Enables debugging messages.  Argument specifies the '
-                      'level of messages to display (e.g., "--debug-level 5" displays '
-                      'all debugging messages at level 5 or below.)') ),
+      CommandOption ('silent', category = "base",
+        help = _('Displays no output at all.')),
 
-         CommandOption('debug-file', category="base", argument=_("filename"),
-              help=_('Sends all debugging messages to a specified file '
-                     '(e.g., "--debug-file trace.log" sends all output to "trace.log")') ),
+      CommandOption ('help', category = "base", action = self.printHelp,
+        help = _('Displays this help screen.')),
 
-         CommandOption('help', category="base",  action=self.printHelp,
-              help=_('Displays this help screen.') ),
+      CommandOption ('help-config', category = "base",
+        action = self.doHelpConfig,
+        help = _('Displays a list of valid configuration file entries, their '
+                 'purpose, and their default values.')),
 
-         CommandOption('help-config', category="base",  action=self.doHelpConfig,
-              help=_('Displays a list of valid configuration file entries, their '
-                     'purpose, and their default values.') ),
+      ##
+      ## Developer options
+      ##
 
-         ##
-         ## Developer options
-         ##
+      CommandOption ('help-dev', category = "base", action = self.printHelpDev,
+        help = _('Display all options of interest to core developers. ')),
 
-         CommandOption('help-dev', category="base", action=self.printHelpDev,
-              help=_('Display all options of interest to core developers. ') ),
+      CommandOption ('selfdoc', category = "dev", action = self.doSelfDoc,
+        argument = _("type[,subtype]"),
+        help = _('Generates self-documentation.')),
 
-         CommandOption('selfdoc', category="dev",  action=self.doSelfDoc,
-              argument=_("type[,subtype]"),
-              help=_('Generates self-documentation.') ),
+      CommandOption ('selfdoc-format', category = "dev", argument = _("format"),
+        help = _('Format to output the self-documentation in. Supported '
+                 'formats are dependent on the type of selfdoc being '
+                 'created.')),
 
-         CommandOption('selfdoc-format', category="dev", argument=_("format"),
-              help=_('Format to output the self-documentation in. Supported formats '
-                     'are dependent on the type of selfdoc being created.') ),
+      CommandOption ('selfdoc-file', category = "dev", argument = _("filename"),
+        help = _('Specifies the filename that selfdoc should write to. If not '
+                 'provided, output is sent to stdout.')),
 
-         CommandOption('selfdoc-file', category="dev", argument=_("filename"),
-              help=_('Specifies the filename that selfdoc should write to. If not provided, output is sent to stdout.') ),
+      CommandOption ('selfdoc-options', category = "dev",
+        argument = _("options"),
+        help = _('Options specific to individual selfdoc types.')),
 
-         CommandOption('selfdoc-options', category="dev", argument=_("options"),
-              help=_('Options specific to individual selfdoc types.') ),
+      CommandOption ('profile', category = "dev",
+        help = _("Run Python's built-in profiler and display the resulting "
+                 "run statistics.")),
 
-         CommandOption('profile', category="dev",
-              help=_("Run Python's built-in profiler and display the resulting "
-                      "run statistics.") ),
-
-         CommandOption('interactive-debugger', category="dev",
-              help=_("Run the app inside Python's built-in debugger ")),
+      CommandOption ('interactive-debugger', category = "dev",
+        help = _("Run the app inside Python's built-in debugger ")),
     ]
 
     if self.USE_DATABASE_OPTIONS:
       self.USE_CONNECTIONS = 1
-      self._base_options += [
+      self._base_options.extend ([ \
+        CommandOption ('username', category = "connections", default = '',
+          argument = 'name',
+          help = _('Username used to log into the database.  Note that if '
+                   'specified, this will be used for all databases.  If not '
+                   'supplied, the program will prompt for username.')),
 
-        CommandOption('username', category="connections", default='',
-              argument='name',
-              help=_('Username used to log into the database.  Note that if '
-                     'specified, this will be used for all databases.  If not '
-                     'supplied, the program will prompt for username.') ),
-
-        CommandOption('password', category="connections", default='',
-              argument='passwd',
-              help=_('Password used to log into the database.  Note that if '
-                     'specified, this will be used for all databases.  If not '
-                     'supplied, the program will prompt for password if needed.'
-                     '\nNOTE: SUPPLYING A PASSWORD VIA THE COMMAND LINE MAY BE '
-                     'CONSIDERED A SECURITY RISK AND IS NOT RECOMMENDED.)') )
-      ]
+        CommandOption ('password', category = "connections", default = '',
+          argument = 'passwd',
+          help = _('Password used to log into the database.  Note that if '
+                   'specified, this will be used for all databases.  If not '
+                   'supplied, the program will prompt for password if needed.'
+                   '\nNOTE: SUPPLYING A PASSWORD VIA THE COMMAND LINE MAY BE '
+                   'CONSIDERED A SECURITY RISK AND IS NOT RECOMMENDED.'))])
 
     if self.USE_CONNECTIONS:
       self._base_options += [
-        CommandOption('help-connections', category="base", action=self.printHelpConn,
-              help=_('Display help information related to database '
-                     'connections, including a list of available drivers.') ),
+        CommandOption ('help-connections', category = "base",
+          action = self.printHelpConn,
+          help = _('Display help information related to database '
+                   'connections, including a list of available drivers.')),
 
-        CommandOption('connections', category="connections",
-              argument="loc",
-              help=_('Specifies the location of the connection definition file. '
-                     '<loc> may specify a file name '
-                     '(/usr/local/gnue/etc/connections.conf),'
-                     'or a URL location '
-                     '(http://localhost/connections.conf).'
-                     'If this option is not specified, the environent variable '
-                     'GNUE_CONNECTIONS is checked.'
-                     'If neither of them is set, "%s" is used as a default.') %
+        CommandOption ('connections', category = "connections",
+          argument = "loc",
+          help = _('Specifies the location of the connection definition file. '
+                   '<loc> may specify a file name '
+                   '(/usr/local/gnue/etc/connections.conf),'
+                   'or a URL location '
+                   '(http://localhost/connections.conf).'
+                   'If this option is not specified, the environent variable '
+                   'GNUE_CONNECTIONS is checked.'
+                   'If neither of them is set, "%s" is used as a default.') %
              os.path.join (paths.config, "connections.conf")) ]
 
     # Python version check
-    try:
-      if sys.hexversion < 0x02010000:
-        self.handleStartupError (_('This application requires Python 2.1 or greater.  You are running Python %s') % sys.version[:5])
-    except AttributeError:
-      # Really, really old version of Python...
-      self.handleStartupError (_('This application requires Python 2.1 or greater.'))
+    if not hasattr (sys, 'hexversion') or sys.hexversion < 0x02010000:
+      msg = _("This application requires Python 2.1 or greater.")
+      if hasattr (sys, 'version'):
+        msg = u_("This application requires Python 2.1 or greater. "
+                 "You are running Python %s") % sys.version [:5]
+
+      raise errors.AdminError, msg
 
 
     #
@@ -222,9 +235,10 @@ class GBaseApp:
       sys.argv=[]
 
     try:
-      opt, self.ARGUMENTS = getopt.getopt(sys.argv[1:], shortoptions, longoptions)
+      opt, self.ARGUMENTS = getopt.getopt (sys.argv[1:], shortoptions,
+                                           longoptions)
     except getopt.error, msg:
-      self.handleStartupError(msg)
+      raise StartupError, "%s" % msg
 
     pendingActions = []
     for o in opt:
@@ -244,6 +258,26 @@ class GBaseApp:
 
     self._run = self.run
 
+    # Are we silent?
+    if self.OPTIONS['silent']:
+      # our file objects (/dev/null and nul) has no encoding, unlike stdout...
+      import __builtin__
+      __builtin__.__dict__['u_'] = __builtin__.__dict__['_']
+      if os.name == 'posix':
+        sout = open('/dev/null','w')
+        serr = open('/dev/null','w')
+      elif os.name == 'nt':
+        sout = open('nul', 'w')
+        serr = open('nul', 'w')
+
+      try:
+        sys.stdout.close()
+        sys.stdout = sout
+        sys.stderr.close()
+        sys.stderr = serr
+      except:
+        pass
+
     # Should we profile?
     if self.OPTIONS['profile']:
       self.run = self._profile
@@ -254,30 +288,24 @@ class GBaseApp:
       self.run = self._debugger
 
     try:
-      GDebug.setDebug(int("%s" % self.OPTIONS['debug-level']),
+      GDebug.setDebug("%s" % self.OPTIONS['debug-level'],
           self.OPTIONS['debug-file'])
     except ValueError:
-      self.handleStartupError(_('The debug_level option ("-d") expects a numerical value.'))
+       raise StartupError, \
+           _('The debug_level option ("-d") expects numerical values.')
 
-
-
-    GDebug.printMesg(1,"Python %s" % sys.version)
-    GDebug.printMesg(1,"Run Options: %s" % opt)
-    GDebug.printMesg(1,"Run Arguments: %s" % self.ARGUMENTS)
+    gDebug (2, "Python %s" % sys.version)
+    gDebug (2, "Run Options: %s" % opt)
+    gDebug (2, "Run Arguments: %s" % self.ARGUMENTS)
 
     # Read the config files
     if application:
       try:
-        self.configurationManager = GConfig.GConfig(application, defaults, configFilename=self.CONFIGFILE)
+        self.configurationManager = GConfig.GConfig (application, self.configDefaults,
+                                              configFilename = self.CONFIGFILE)
       except ConfigParser.NoSectionError, msg:
-        self.handleStartupError(_('The gnue.conf file is incomplete: ')\
-                                + '\n   %s'         % msg)
-      except Exception, msg:
-        etype=string.splitfields(str(sys.exc_type),'.')
-        etype.reverse()
-        self.handleStartupError( \
-          _('%s while reading gnue.conf: ') % etype[0] \
-          + '\n   %s'         % msg)
+        raise errors.AdminError, \
+            u_("The gnue.conf file is incomplete:\n   %s") % msg
 
     # Add custom import to python's namespace
     try:
@@ -292,7 +320,7 @@ class GBaseApp:
 
     # Get the connection definitions
     if connections != None:
-      GDebug.printMesg(1,"Reusing connections instance")
+      gDebug(7,"Reusing connections instance")
       self.connections = connections
     elif self.USE_CONNECTIONS:
 
@@ -310,28 +338,21 @@ class GBaseApp:
         self.connections_file = os.environ['GNUE_CONNECTIONS']
       else:
         self.connections_file = os.path.join (paths.config, "connections.conf")
-#        self.handleStartupError(
-#            _('Unable to load the connections definition file.\n') \
-#            + _('\n   Please set the environmental variable GNUE_CONNECTIONS or ') \
-#            + _('\n   use the "-f" command option.'))
 
-      GDebug.printMesg(1, 'Connection Definition: "%s"' % self.connections_file)
+      gDebug(2, 'Connection Definition: "%s"' % self.connections_file)
 
       try:
-        self.connections = GConnections.GConnections(self.connections_file, loginOptions=lhOptions)
+        self.connections = GConnections.GConnections (self.connections_file,
+                                                      loginOptions = lhOptions)
       except GConnections.InvalidFormatError, msg:
-        self.handleStartupError(
-            _('Unable to load the connections definition file.\n') \
-            + _('\n   The connections file is in an invalid format. ') \
-            + '\n   %s' \
-               % msg)
-      except IOError:
-        self.handleStartupError(
-             _('Unable to load the connections definition file.\n') \
-             + _('\n   The connections file specified either does ') \
-             + _('\n   not exist or is not readable by your account.\n') \
-             + _('\n   Location: "%s"')
-                % self.connections_file)
+        raise errors.AdminError, \
+            u_("Unable to load the connections definition file.\n\n"
+               "The connections file is in an invalid format.\n%s") % msg
+
+      except IOError, err:
+        raise StartupError, \
+            u_("Unable to load the connections definition file: %s.") \
+            % self.connections_file
 
   def addCommandOption(self, *args, **parms):
     self.COMMAND_OPTIONS.append(CommandOption(*args, **parms))
@@ -341,8 +362,10 @@ class GBaseApp:
   #
   def printVersion(self):
     from gnue.common import VERSION as commonVersion
-    print _("\n%s\nVersion %s\n") % (self.NAME, self.VERSION)
-    print _("GNUe Common Version %s\n") % commonVersion
+    print o(u_("\n%(name)s\nVersion %(version)s\n") \
+             % {'name': self.NAME, 'version': self.VERSION})
+    print o(u_("GNUe Common Version %s\n") % commonVersion)
+
 
   #
   #  "Help" helpers
@@ -481,11 +504,11 @@ class GBaseApp:
     for param in paramList:
       psplit = string.split(param,'=',1)
       if len(psplit) == 1:
-        self.handleStartupError ( \
-                                  'Parameter "%s" specified, but no value supplied.' % psplit[0] )
+        raise StartupError, \
+            'Parameter "%s" specified, but no value supplied.' % psplit[0]
       parameters[string.lower(psplit[0])] = psplit[1]
 
-      GDebug.printMesg(2,'Param "%s"="%s" ' % \
+      gDebug(2,'Param "%s"="%s" ' % \
                          (string.lower(psplit[0]), psplit[1]))
     return parameters
 
@@ -499,10 +522,10 @@ class GBaseApp:
     # if msg is multiline, then surround with dashes to set it apart
     if string.find("%s" % msg, "\n") + 1:
       print '-' * 60
-    print _("Error: %s") % msg
+    print o(u_("Error: %s") % msg)
     if string.find("%s" % msg, "\n") + 1:
       print '-' * 60
-    print _("\nFor help, type:\n   %s --help\n") % (self.COMMAND)
+    print o(u_("\nFor help, type:\n   %s --help\n") % self.COMMAND)
     sys.exit()
 
 
@@ -559,5 +582,41 @@ class GBaseApp:
 
   def doHelpConfig(self):
     self.printHelpHeader()
-    print GConfig.printableConfigOptions(defaults)
+    print GConfig.printableConfigOptions(self.configDefaults)
     sys.exit()
+
+
+  # ---------------------------------------------------------------------------
+  # Catch an exception
+  # ---------------------------------------------------------------------------
+
+  def excepthook (self, etype, value, traceback):
+    """
+    This function catches an exception and evaluates it using getException ().
+    The exception-tuple is then passed to showException (), which might get
+    overriden by a descendant.
+    """
+    sys.excepthook = sys.__excepthook__
+    self._showException (*errors.getException (None, etype, value, traceback))
+    sys.excepthook = self.excepthook
+
+
+  # ---------------------------------------------------------------------------
+  # Show an exception
+  # ---------------------------------------------------------------------------
+
+  def _showException (self, group, name, message, detail):
+    """
+    This function shows an exception specified by the given parameters.
+    @param group: Exception group ('system', 'admin', 'application', 'user')
+    @param name: Name of the exception
+    @param message: Message of the exception
+    @param detail: Detail of the exception (usually holding a traceback)
+    """
+
+    if group in ['user', 'admin']:
+      sys.__stderr__.write ("%s: %s\n" % (self.COMMAND, o(message)))
+      sys.__stderr__.write ("%s\n" % \
+                           o(u_("For help, type: %s --help") % self.COMMAND))
+    else:
+      sys.stderr.write ("%s\n" % o(detail))
