@@ -16,7 +16,7 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# Copyright 2000-2004 Free Software Foundation
+# Copyright 2000-2005 Free Software Foundation
 #
 # FILE:
 # __init__.py
@@ -32,9 +32,28 @@
 # For Python 2.0/2.1, we add them ourselves
 import sys
 if sys.hexversion < 0x02020000:
-  __builtins__ ['True']  = 1
-  __builtins__ ['False'] = 0
+  __builtins__['True']  = 1
+  __builtins__['False'] = 0
+
+# Add "basestring" for Python 2.2 or less
+if sys.hexversion < 0x02030000:
+  __builtins__['basestring'] = type("")
 
 import GDebug
 import i18n
 import checktype
+import errors
+
+# Add gStartupStatus global for startup status dialogs
+__builtins__['gStartupStatus'] = lambda string:string
+
+
+# Add the decimal class for python 2.3 or less
+#if sys.hexversion < 0x02040000:
+#  pth = sys.path
+#  from gnue.common.external import decimal
+#  sys.modules['decimal'] = decimal
+#else:
+#  import decimal
+#
+#__builtins__['Decimal'] = decimal.Decimal
