@@ -16,7 +16,7 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# Copyright 2002-2004 Free Software Foundation
+# Copyright 2002-2005 Free Software Foundation
 #
 # FILE:
 # __init__.py
@@ -30,39 +30,40 @@
 from constants import *
 
 __classes__ = [
-  "sio ",
-  "Button ",
-  "CheckBox ",
-  "CheckBoxGroup ",
-  "ComboBox ",
-  "Control ",
-  "Dialog ",
-  "DirChooser ",
-  "FileSelectPad ",
-  "FileSelector ",
-  "FocusedLabel ",
-  "Frame ",
-  "GraphicObject ",
-  "GroupBox ",
-  "InputBox ",
-  "Label ",
-  "Menu ",
-  "MenuBar ",
-  "MenuItem ",
-  "MsgBoxOK ",
-  "MsgBoxYesNo ",
-  "Progress ",
-  "RadioButton ",
-  "RadioGroup ",
-  "ScreenContainer ",
-  "ScrollBar ",
-  "StatusBar ",
-  "TextBox ",
-  "TextArea ",
-  "VButton ",
+  "sio",
+  "Application",
+  "Button",
+  "CheckBox",
+  "CheckBoxGroup",
+  "ComboBox",
+  "Container",
+  "Control",
+  "Dialog",
+  "DirChooser",
+  "FileSelectPad",
+  "FileSelector",
+  "FocusedLabel",
+  "Frame",
+  "GraphicObject",
+  "GroupBox",
+  "InputBox",
+  "Label",
+  "Menu",
+  "MenuBar",
+  "MenuItem",
+  "MsgBoxOK",
+  "MsgBoxYesNo",
+  "Progress",
+  "RadioButton",
+  "RadioGroup",
+  "ScrollBar",
+  "StatusBar",
+  "TextBox",
+  "TextArea",
+  "VButton",
   "VScrollBar "
 ]
 
 for module in  __classes__ :
   exec "from %s import %s" % (module, module)
-    
+
