@@ -16,7 +16,7 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# Copyright 2000-2004 Free Software Foundation
+# Copyright 2000-2005 Free Software Foundation
 #
 # FILE:
 # GDataObjects.py
@@ -27,13 +27,8 @@
 #
 # NOTES:
 #
-# TODO: Generate warnings whenever anything in this file is accessed.
-#
 
 from Exceptions import *
 from drivers.Base import *
 
-
-# Used to store schema data
-# TODO: Is this used anywhere??
-import string
+print "Warning: Accessing obsolete module 'GDataObjects'"
