@@ -16,15 +16,8 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# Copyright 2000-2004 Free Software Foundation
+# Copyright 2000-2005 Free Software Foundation
 #
 """
 Database-specific datasource drivers
 """
-DRIVERS = [ 'adodbapi','db2','informix','mysql','postgresql',
-            'sqlite','dbf','ingres','odbc','sapdb',
-            'sqlrelay','appserver','gadfly','interbase',
-            'oracle','sybase','ldap','special','csv' ]
-
-ALIASES = { 'firebird': 'interbase',
-            'maxdb': 'sapdb' }
