@@ -16,7 +16,7 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# Copyright 2000-2004 Free Software Foundation
+# Copyright 2000-2005 Free Software Foundation
 #
 """
 Information on this driver
@@ -56,6 +56,5 @@ Examples
 
 Notes
 -----
-1. This driver has been partially tested.  If you are successfully using
-   this driver, please let us know.
+1. This driver is currently unmaintained and not functional.
 """
