@@ -16,7 +16,7 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# Copyright 2000-2004 Free Software Foundation
+# Copyright 2000-2005 Free Software Foundation
 #
 # FILE:
 # interbase/ResultSet.py
@@ -38,4 +38,13 @@ from gnue.common.datasources.drivers import DBSIG2
 #
 #
 class ResultSet(DBSIG2.ResultSet):
-  pass
+
+  def __init__(self, *args, **parms):
+    self.conditions = parms['conditions']
+    self.sql = parms['sql']
+    del parms['conditions']
+    del parms['sql']
+    
+    DBSIG2.ResultSet.__init__(self, *args, **parms)
+
+    self._recordCount = self._dataObject._getQueryCount(self.conditions, self.sql)
