@@ -16,7 +16,7 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# Copyright 2000-2004 Free Software Foundation
+# Copyright 2000-2005 Free Software Foundation
 #
 # FILE:
 # datasources/drivers/ldap/Driver.py
@@ -120,7 +120,7 @@ class LDAP_DataConnection:
 
   def remove(self,dn):
     
-    GDebug.printMesg(6, 'remove entry dn=%s' % dn)
+    GDebug.printMesg(9, 'remove entry dn=%s' % dn)
     
     self._ldapCon.delete_s(dn)
     
@@ -138,7 +138,7 @@ class LDAP_DataConnection:
     # create new DN
     dn="%s=%s,%s,%s" % (pk_field,pk_value,searchdn,self._basedn)
       
-    GDebug.printMesg(6, 'add new entry dn=%s fields=%s' % (dn,modlist))
+    GDebug.printMesg(9, 'add new entry dn=%s fields=%s' % (dn,modlist))
     
     self._ldapCon.add_s(dn,modlist)      
 
@@ -147,7 +147,7 @@ class LDAP_DataConnection:
       # build new DN
       newdn="%s=%s,%s,%s" % (pk_field,pk_value,searchdn,self._basedn)
       
-      GDebug.printMesg(6, 'FQDN rename operation change fqdn=%s -> %s' % \
+      GDebug.printMesg(9, 'FQDN rename operation change fqdn=%s -> %s' % \
                        (dn,newdn))
       
       dataCon.modrdn_s(dn,newdn)
@@ -165,7 +165,7 @@ class LDAP_DataConnection:
       modlist.append((ldap.MOD_REPLACE,f,values))
 
       
-    GDebug.printMesg(6, 'modified entry dn=%s fields=%s' % (dn,modlist))
+    GDebug.printMesg(9, 'modified entry dn=%s fields=%s' % (dn,modlist))
     
     self._ldapCon.modify_s(dn,modlist)      
 
@@ -192,7 +192,7 @@ class LDAP_RecordSet(GDataObjects.RecordSet):
         return ""
     
     
-  def _postChanges(self):
+  def _postChanges(self, recordNumber=None):
     
     update_dict={}
     dataCon=self._parent._dataObject._dataConnection
@@ -203,7 +203,7 @@ class LDAP_RecordSet(GDataObjects.RecordSet):
     if 1==1:
 #    try:
       if self._deleteFlag:
-        GDebug.printMesg(5, 'LDAP database driver: Instance deleted')
+        GDebug.printMesg(9, 'LDAP database driver: Instance deleted')
         # TODO: check result
         dataCon.remove(self._fqdn)
       
@@ -218,7 +218,7 @@ class LDAP_RecordSet(GDataObjects.RecordSet):
                        self._fields[primarykey],
                        searchdn)
         
-        GDebug.printMesg(5, 'LDAP database driver: new Instance ' + \
+        GDebug.printMesg(9, 'LDAP database driver: new Instance ' + \
                          'created and inserted')
       
       elif self._updateFlag:
@@ -244,7 +244,7 @@ class LDAP_RecordSet(GDataObjects.RecordSet):
                        self._fields[primarykey],
                        searchdn)
                   
-        GDebug.printMesg(5, 'LDAP database driver: Instance updated')
+        GDebug.printMesg(9, 'LDAP database driver: Instance updated')
         
 
 ##    except ldap.INVALID_SYNTAX:
@@ -277,7 +277,7 @@ class LDAP_ResultSet(GDataObjects.ResultSet):
     # Populate field names
     self._fieldNames = self._dataObject.getFields(self._dataObject.table)           
 
-    GDebug.printMesg(5, 'ResultSet created')
+    GDebug.printMesg(9, 'ResultSet created')
 
 
   def _loadNextRecord(self):
@@ -291,7 +291,7 @@ class LDAP_ResultSet(GDataObjects.ResultSet):
       except IndexError,TypeError:
         return 
       
-      GDebug.printMesg(8, 'Add record %s with data %s' % (dn,data))
+      GDebug.printMesg(9, 'Add record %s with data %s' % (dn,data))
       
       record=self._recordSetClass(parent=self,initialData=data)
       
@@ -381,13 +381,13 @@ class LDAP_DataObject(GDataObjects.DataObject):
   def __init__(self):
     GDataObjects.DataObject.__init__(self)
 
-    GDebug.printMesg (1,"LDAP database driver backend initializing")
+    GDebug.printMesg (9,"LDAP database driver backend initializing")
     self._resultSetClass = LDAP_ResultSet
     self._DatabaseError = ldap.LDAPError
 
   def connect(self, connectData={}):
     
-    GDebug.printMesg(1,"LDAP database driver connecting...")
+    GDebug.printMesg(9,"LDAP database driver connecting...")
     
     try:
 
@@ -395,14 +395,14 @@ class LDAP_DataObject(GDataObjects.DataObject):
 
         uri = "ldap://%s:%s/" % (connectData['host'],connectData['port'])
 
-        GDebug.printMesg(3,"Initialize connection to %s." % uri)
+        GDebug.printMesg(9,"Initialize connection to %s." % uri)
 
         ldapCon = ldap.initialize(uri,
                                   trace_level=0,
                                   trace_file=sys.stdout)
       else:
         
-        GDebug.printMesg(3,"Open connection to %s." % connectData['host'])
+        GDebug.printMesg(9,"Open connection to %s." % connectData['host'])
 
         if connectData.has_key("port"):
           
@@ -412,7 +412,7 @@ class LDAP_DataObject(GDataObjects.DataObject):
           
           ldapCon = ldap.open(connectData['host'])
       
-      GDebug.printMesg (5,"Connection established.")
+      GDebug.printMesg (9,"Connection established.")
       
     except ldap.LDAPError , msg:
       
@@ -424,9 +424,9 @@ class LDAP_DataObject(GDataObjects.DataObject):
       #  a the moment python-ldap supports SIMPLE_AUTH only
       #
       
-      GDebug.printMesg(3,"Authentificate against LDAP server")
+      GDebug.printMesg(9,"Authentificate against LDAP server")
       
-      GDebug.printMesg(3,"User '%s' Password '%s' " % \
+      GDebug.printMesg(9,"User '%s' Password '%s' " % \
                        (connectData['_username'],
                         connectData['_password']))
       
@@ -439,7 +439,7 @@ class LDAP_DataObject(GDataObjects.DataObject):
         ldapCon.simple_bind_s(connectData['_username'],
                               connectData['_password'])
         
-      GDebug.printMesg (5,"Successfully bound to LDAP server.")
+      GDebug.printMesg (9,"Successfully bound to LDAP server.")
 
     except:
       
@@ -480,7 +480,7 @@ class LDAP_DataObject(GDataObjects.DataObject):
         
       else:
         scope=ldap.SCOPE_ONELEVEL
-        GDebug.printMesg(3,"Wrong setting for scope parameter '%s'"\
+        GDebug.printMesg(1,"Wrong setting for scope parameter '%s'"\
                          % connectData['scope'])
         
     self._dataConnection = LDAP_DataConnection(ldapCon,
@@ -506,7 +506,7 @@ class LDAP_DataObject(GDataObjects.DataObject):
 
     query = self.__conditionToLDAPQuery(None,cond._children[0])
     
-    GDebug.printMesg(7,'Full query in : %s' % query)
+    GDebug.printMesg(3,'Full query in : %s' % query)
     
     return query
 
@@ -548,13 +548,19 @@ class LDAP_DataObject(GDataObjects.DataObject):
           # check for too less parameters 
           if len(conditionTree._children) < self.conditionElements[otype][0]:
             raise GConditions.ConditionError, \
-                  _('Condition element "%s" expects at least %s arguments; found %s') % \
-                  (otype, self.conditionElements[otype][0], len(conditionTree._children))
+                  u_('Condition element "%(element)s" expects at least '
+                     '%(expected)s arguments; found %(found)s') \
+                  % {'element' : otype,
+                     'expected': self.conditionElements[otype][0],
+                     'found'   : len (conditionTree._children)}
           # check for too much parameters 
           if len(conditionTree._children) > self.conditionElements[otype][1]:
             raise GConditions.ConditionError, \
-                  _('Condition element "%s" expects at most %s arguments; found %s') % \
-                  (otype, self.conditionElements[otype][1], len(conditionTree._children))
+                  u_('Condition element "%(element)s" expects at most '
+                     '%(expected)s arguments; found %(found)s') \
+                  % {'element' : otype,
+                     'expected': self.conditionElements[otype][1],
+                     'found'   : len (conditionTree._children)}
           # check
           if len(conditionTree._children) > self.conditionElements[otype][0]:
             return self.conditionElements[otype][2] % \
@@ -564,7 +570,7 @@ class LDAP_DataObject(GDataObjects.DataObject):
                  tuple(conditionTree._children)
         else:
           raise GConditions.ConditionNotSupported, \
-                _('Condition clause "%s" is not supported by this db driver.') % otype
+                u_('Condition clause "%s" is not supported by this db driver.') % otype
 
   def computeSearchDN(self, tablename):
     #
@@ -595,7 +601,7 @@ class LDAP_DataObject(GDataObjects.DataObject):
       seperator=", "
       count = count - 2
 
-    GDebug.printMesg (5,"Converting tablename '%s' into search DN '%s'." % \
+    GDebug.printMesg (9,"Converting tablename '%s' into search DN '%s'." % \
                       (tablename,searchdn))
   
     return searchdn
@@ -613,24 +619,24 @@ class LDAP_DataObject(GDataObjects.DataObject):
     filter   = ""
     
     if conditions:
-      GDebug.printMesg (5,"Setting Conditions ...")
+      GDebug.printMesg (9,"Setting Conditions ...")
       
       filter=self._buildQuery(conditions)
       
-      GDebug.printMesg (6," %s -> %s" % (conditions, filter))
+      GDebug.printMesg (9," %s -> %s" % (conditions, filter))
 
     if hasattr(self,"order_by"):
-      GDebug.printMesg (5,"FIXME: Sorting for LDAP not implemented "+\
+      GDebug.printMesg (1,"FIXME: Sorting for LDAP not implemented "+\
                         "(order_by='%s')" % self.order_by)
 
     if not hasattr(self,"primarykey"):
       self.primarykey = 'cn'
       
-    GDebug.printMesg (5,"Use '%s' as FQDN base field" %\
+    GDebug.printMesg (9,"Use '%s' as FQDN base field" %\
                       self.primarykey)
 
     try:
-      GDebug.printMesg (0,"Started search for matching LDAP entries " +\
+      GDebug.printMesg (9,"Started search for matching LDAP entries " +\
                         "(DN=%s,Filter=%s)" % (searchdn , filter))
 
       cursor=self._dataConnection.search(searchdn, filter, self.primarykey)
@@ -655,10 +661,10 @@ class LDAP_DataObject(GDataObjects.DataObject):
   # through the _post method of the RecordSet class. 
   
   def commit(self): 
-    GDebug.printMesg (5,"LDAP database driver: fake commit")
+    GDebug.printMesg (9,"LDAP database driver: fake commit")
 
   def rollback(self): 
-    GDebug.printMesg (5,"LDAP database driver: fake rollback")
+    GDebug.printMesg (9,"LDAP database driver: fake rollback")
 
 
 
