@@ -16,7 +16,7 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# Copyright 2000-2004 Free Software Foundation
+# Copyright 2000-2005 Free Software Foundation
 #
 
 # Indicate that this is no plugin
@@ -24,6 +24,4 @@ __noplugin__ = True
 
 from Connection import *
 from DataObject import *
-from ResultSet import *
-from RecordSet import *
 
