@@ -16,7 +16,7 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place 
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# Copyright 2001-2004 Free Software Foundation
+# Copyright 2001-2005 Free Software Foundation
 #
 # FILE:
 # _static/DBdriver.py
@@ -30,8 +30,7 @@
 
 __all__ = ['STATIC_DataObject']
 
-from gnue.common.apps import GDebug
-from gnue.common.datasources.GDataObjects import DataObject
+from gnue.common.datasources.drivers.Base import DataObject
 from ResultSet import STATIC_ResultSet
 
 class STATIC_DataObject (DataObject): 
