@@ -16,7 +16,7 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place 
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# Copyright 2001-2004 Free Software Foundation
+# Copyright 2001-2005 Free Software Foundation
 #
 # FILE:
 # _static/DBdriver.py
@@ -30,20 +30,15 @@
 
 __all__ = ['STATIC_RecordSet']
 
-from gnue.common.apps import GDebug
 import string
-from gnue.common.datasources.GDataObjects import *
+from gnue.common.datasources.drivers import Base
 
 
-class STATIC_RecordSet (RecordSet): 
+class STATIC_RecordSet (Base.RecordSet): 
 
   def isPending(self):
     return 0
 
   # Post any changes to database
-  def _postChanges(self): 
+  def _postChanges(self, recordNumber=None): 
     return 1
-
-  
-
-
