@@ -16,14 +16,15 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# Copyright 2000-2004 Free Software Foundation
+# Copyright 2000-2005 Free Software Foundation
 #
 # FILE:
 # GParser.py
 #
 # DESCRIPTION:
-# Class that contains a SAX2-based XML processor for GNUe
-#
+"""
+Class that contains a SAX2-based XML processor for GNUe
+"""
 # NOTES:
 #
 
@@ -49,7 +50,9 @@ except ImportError:
 
 
 import string
-from gnue.common.apps import GDebug
+import os
+
+from gnue.common.apps import errors
 from gnue.common.formatting import GTypecast
 from gnue.common.definitions.GParserHelpers import GContent
 
@@ -58,8 +61,20 @@ from gnue.common.definitions.GParserHelpers import GContent
 #
 # Error classed raised for markup errors
 #
-class MarkupError(StandardError):
-  pass
+class MarkupError(errors.ApplicationError):
+  def __init__ (self, message, url = None, line = None):
+    errors.ApplicationError.__init__ (self, message)
+    text = [message]
+    if url is not None:
+      if line is not None:
+        msg = u_("XML markup error in '%(url)s' at line %(line)s:") \
+              % {'url': url, 'line': line}
+      else:
+        msg = u_("XML markup error in '%(url)s':") % {'url': url}
+
+      text.insert (0, msg)
+
+    self.detail = string.join (text, os.linesep)
 
 
 #######################################################
@@ -83,7 +98,7 @@ class MarkupError(StandardError):
 #######################################################
 
 def loadXMLObject(stream, handler, rootType, xmlFileType,
-  initialize=1, attributes={}, initParameters={}):
+    initialize=True, attributes={}, initParameters={}, url = None):
 
   # Create a parser
   parser = xml.sax.make_parser()
@@ -99,25 +114,42 @@ def loadXMLObject(stream, handler, rootType, xmlFileType,
   object = None
 
   # Create the handler
-  dh = handler()
+  dh = handler ()
+
+  # pass the url of the stream and a pointer to the parser instance down to the
+  # handler, so it's able to do better error reporting
+  if url is None:
+    url = hasattr (stream, 'url') and stream.url or '[unknown]'
+
+  dh.url = url
+  dh.parser = parser
+
   dh.initValidation()
 
   # Tell the parser to use our handler
-  parser.setContentHandler(dh)  
+  parser.setContentHandler(dh)
+  
+  try:
+    parser.parse (stream)
 
-  parser.parse(stream)
+  except xml.sax.SAXParseException, e:
+    raise MarkupError, (errors.getException () [2], url, e.getLineNumber ())
 
   object = dh.getRoot()
 
   if not object:
-    tmsg = _("Error loading %s: empty definition file") % (xmlFileType)
-    raise MarkupError, tmsg
-  elif object._type != rootType:
-    tmsg = _("Error loading %s: not a valid %s definition (expected: %s, got: %s)") % \
-        (xmlFileType, xmlFileType, rootType, object._type) 
-    raise MarkupError, tmsg
+    tmsg = u_("Error loading %s: empty definition file") % (xmlFileType)
+    raise MarkupError, (tmsg, url)
 
-  dh.finalValidation()
+  elif object._type != rootType:
+    tmsg = u_("Error loading %(filetype)s: not a valid %(filetype)s definition "
+              "(expected: %(expected)s, got: %(got)s)") \
+           % {'filetype': xmlFileType,
+              'expected': rootType,
+              'got'     : object._type}
+    raise MarkupError, (tmsg, url)
+
+  dh.finalValidation ()
 
   # Set the root object's attributes
   #
@@ -130,7 +162,7 @@ def loadXMLObject(stream, handler, rootType, xmlFileType,
   object.walk(addAttributesWalker,attributes=attributes)
 
   if initialize:
-    GDebug.printMesg(10,"Initializing the object tree starting at %s" %(object))
+    gDebug (7, "Initializing the object tree starting at %s" %(object))
     object.phaseInit(dh._phaseInitCount)
 
 
@@ -174,15 +206,29 @@ def default(attrs, key, default):
 #
 #######################################################
 class xmlHandler(xml.sax.ContentHandler):
+
+  # Set to default namespace (which would be dropped)
+  # i.e., if default namespace is "Foo", then <Foo:test> would
+  # just be processed as <test>.
+  default_namespace = None
+
+  # Drop any elements that have a namespaces we don't know about?
+  ignore_unknown_namespaces = False
+
   def __init__(self):
 
     self.xmlElements = {}
-    self.xmlMasqueradeNamespaceElements = None
-    self.xmlNamespaceAttributesAsPrefixes = 0
 
+    # This is a hack, currently used only by GRPassThru
+    self.xmlMasqueradeNamespaceElements = None
+
+    # Use namespace as a prefix in GObjects
+    self.xmlNamespaceAttributesAsPrefixes = False
+
+    # Internal stuff
     self.xmlStack = []
     self.nameStack = []
-    self.bootstrapflag = 0
+    self.bootstrapflag = False
     self.uniqueIDs = {}
     self.root = None
     self._phaseInitCount = 0
@@ -190,6 +236,7 @@ class xmlHandler(xml.sax.ContentHandler):
     self._requiredTags = []
     self._singleInstanceTags = []
     self._tagCounts = {}
+    self.url = None
 
   #
   # Called by client code to get the "root" node
@@ -227,8 +274,8 @@ class xmlHandler(xml.sax.ContentHandler):
 
     for element in self._requiredTags:
       if self._tagCounts[element] < 1:
-        tmsg = _("File is missing required tag <%s>") % (element)
-        raise MarkupError, tmsg
+        tmsg = u_("File is missing required tag <%s>") % (element)
+        raise MarkupError, (tmsg, self.url)
 
 
   #
@@ -241,18 +288,20 @@ class xmlHandler(xml.sax.ContentHandler):
     attrs = {}
     loadedxmlattrs = {}
 
-    if not ns:
+    if not ns or ns == self.default_namespace:
       #
       # No namespace qualifier
       #
-      GDebug.printMesg(50, "<%s>" % name)
+      gDebug (7, "<%s>" % name)
 
       try:
         baseAttrs = self.xmlElements[name].get('Attributes',{})
+
       except KeyError:
-        tmsg = _("Error processing <%s> tag [I do not know what a <%s> tag does]") \
-                % (name, name)
-        raise MarkupError, tmsg
+        raise MarkupError, \
+            (u_("Error processing <%(tagname)s> tag [I do not know what a "
+                "<%(tagname)s> tag does]") % {'tagname': name}, self.url,
+                self.parser.getLineNumber ())
 
       xmlns = {}
 
@@ -262,7 +311,7 @@ class xmlHandler(xml.sax.ContentHandler):
         if attrns:
           if not self.xmlNamespaceAttributesAsPrefixes:
             tmsg = _("Unexpected namespace on attribute")
-            raise MarkupError, tmsg
+            raise MarkupError, (tmsg, self.url, self.parser.getLineNumber ())
           prefix = attrns.split(':')[-1]
           attrs[prefix + '__' + attr] = saxattrs[qattr]
           xmlns[prefix] = attrns
@@ -270,13 +319,15 @@ class xmlHandler(xml.sax.ContentHandler):
         else:
 
           # Typecasting, anyone?  If attribute should be int, make it an int
-          try:            
-            attrs[attr] = baseAttrs[attr].get('Typecast',GTypecast.text)(saxattrs[qattr])
+          try:
+            attrs [attr] = baseAttrs [attr].get ('Typecast', GTypecast.text)(saxattrs[qattr])
             loadedxmlattrs[attr] = attrs[attr]
           except KeyError:
-            tmsg = _('Error processing <%s> tag [I do not recognize the "%s" attribute')\
-                % (name, attr)
-            raise MarkupError, tmsg
+            raise MarkupError, \
+                (u_('Error processing <%(tagname)s> tag [I do not '
+                   'recognize the "%(attribute)s" attribute]') \
+                % {'tagname': name,
+                   'attribute': attr}, self.url, self.parser.getLineNumber ())
 
           # this error shouldn't occure anymore
           #except UnicodeError:
@@ -293,22 +344,38 @@ class xmlHandler(xml.sax.ContentHandler):
           # If this attribute must be unique, check for duplicates
           if baseAttrs[attr].get('Unique',0): # default (baseAttrs[attr],'Unique',0):
             if self.uniqueIDs.has_key('%s' % (saxattrs[qattr])):
-              tmsg = _('Error processing <%s> tag ["%s" attribute should be unique; duplicate value is "%s"]')\
-                % (name, attr, saxattrs[qattr]) 
-              raise MarkupError, tmsg
+              tmsg = u_('Error processing <%(tag)s> tag ["%(attribute)s" '
+                        'attribute should be unique; duplicate value is '
+                        '"%(duplicate)s"]') \
+                     % {'tag'      : name,
+                        'attribute': attr,
+                        'duplicate': saxattrs [qattr]}
+              raise MarkupError, (tmsg, self.url, self.parser.getLineNumber ())
+
+            # FIXME: If we really want to have a working 'Unique'-Attribute, we
+            # would add the following line. But this would break forms atm.
+            # self.uniqueIDs ["%s" % saxattrs [qattr]] = True
+
 
       for attr in baseAttrs.keys():
-        if not attrs.has_key(attr):
-
-          # Pull default values for missing attributes
-          if baseAttrs[attr].has_key ('Default'):
-            attrs[attr] = baseAttrs[attr].get('Typecast', GTypecast.text) (baseAttrs[attr]['Default'])# default(baseAttrs[attr],'Typecast', GTypecast.text) (baseAttrs[attr]['Default'])
-
-          # Check for missing required attributes
-          elif baseAttrs[attr].get('Required', 0): #default(baseAttrs[attr], 'Required', 0):
-            tmsg = _('Error processing <%s> tag [required attribute "%s" not present]')\
-                % (name, attr)
-            raise MarkupError, tmsg
+        try:
+          if not attrs.has_key(attr):
+  
+            # Pull default values for missing attributes
+            if baseAttrs[attr].has_key ('Default'):
+              attrs[attr] = baseAttrs[attr].get('Typecast', GTypecast.text) (baseAttrs[attr]['Default'])
+  
+            # Check for missing required attributes
+            elif baseAttrs[attr].get('Required', False): 
+              tmsg = u_('Error processing <%(tagname)s> tag [required attribute '
+                        '"%(attribute)s" not present]') \
+                    % {'tagname'  : name,
+                        'attribute': attr}
+              raise MarkupError, (tmsg, self.url, self.parser.getLineNumber ())
+        except (AttributeError, KeyError), msg: 
+          raise errors.SystemError, _( 
+            'Error in GParser xmlElement definition for %s/%s'
+                ) % (name, attr) + '\n' + str(msg)
 
       attrs['_xmlnamespaces'] = xmlns
 
@@ -329,7 +396,7 @@ class xmlHandler(xml.sax.ContentHandler):
       # namespace qualifier and we are masquerading
       #
 
-      GDebug.printMesg(50, "<%s:%s>" % (ns,name))
+      gDebug (7, "<%s:%s>" % (ns,name))
 
       for qattr in saxattrs.keys():
         attrns, attr = qattr
@@ -340,25 +407,36 @@ class xmlHandler(xml.sax.ContentHandler):
       try:
         object = self.xmlMasqueradeNamespaceElements(self.xmlStack[0])
       except IndexError:
-        tmsg = _("Error processing <%s:%s> tag: root element needs to be in default namespace") %\
-               (ns,name)
-        raise MarkupError, tmsg
-	    
+        tmsg = u_("Error processing <%(namespace)s:%(name)s> tag: root "
+                  "element needs to be in default namespace") \
+              % {'namespace': ns,
+                  'name'     : name}
+        raise MarkupError, (tmsg, self.url, self.parser.getLineNumber ())
+
       object._xmltag = name
       object._xmlnamespace = ns
       object._listedAttributes = loadedxmlattrs.keys()
 
+    elif self.ignore_unknown_namespaces:
+      self.xmlStack.insert(0, None)
+      self.nameStack.insert(0, None)
+      return
     else:
       #
       # namespace qualifier and we are not masquerading
       #
-      print _("WARNING: Markup includes namespaces, but the current tool does not include namespace support!")
-      sys.exit()
+      tmsg =  u_("WARNING: Markup includes unsupported namespace '%s'." ) % (ns)
+      raise MarkupError, (tmsg, self.url, self.parser.getLineNumber ())
 
 
     # Save the attributes loaded from XML file
     # (i.e., attributes that were not defaulted)
     object._loadedxmlattrs = loadedxmlattrs
+    
+    # We make the url of the xml-stream and the line number of the element
+    # available to the instance (for later error handling)
+    object._lineNumber     = self.parser.getLineNumber ()
+    object._url            = self.url
 
     # Set the attributes
     object.__dict__.update(attrs)
@@ -402,10 +480,15 @@ class xmlHandler(xml.sax.ContentHandler):
     ns, name = qtag
     self.nameStack.pop(0)
     child = self.xmlStack.pop(0)
+
+    if not child:
+      return
+
     inits = child._buildObject()
     self._phaseInitCount = (inits != None and inits > self._phaseInitCount \
                             and inits or self._phaseInitCount)
-    GDebug.printMesg(50, "</%s>" % name)
+    gDebug (7, "</%s>" % name)
+
 
 
 class GImportItem(GObj):
@@ -428,8 +511,18 @@ class GImportItem(GObj):
      # Open the library and convert it into objects
      #
      handle = openResource(self.library)
-     form = self._xmlParser.loadFile(handle, self.findParentOfType(None)._app, initialize=0)
-     handle.close()
+     parent = self.findParentOfType (None)
+
+     # Let the parent provide it's instance either as _app or _instance
+     if hasattr (parent, '_instance'):
+       instance = parent._instance
+     elif hasattr (parent, '_app'):
+       instance = parent._app
+     else:
+       instance = None
+
+     form = self._xmlParser.loadFile (handle, instance, initialize = 0)
+     handle.close ()
      id = 'id'
      if hasattr(self,'name'):
          id = 'name'
@@ -447,12 +540,14 @@ class GImportItem(GObj):
        for key in self._loadedxmlattrs.keys():
          if key[0] != '_':
            rv.__dict__[key] = self._loadedxmlattrs[key]
-           GDebug.printMesg (5, ">>> Moving %s" % key)
+           gDebug (7, ">>> Moving %s" % key)
        rv._buildObject()
      else:
-         tmsg =  _("Unable to find an importable object named %s in %s") \
-            % (self.name, self.library)
-         raise MarkupError, tmsg
+         raise MarkupError, \
+             (u_("Unable to find an importable object named %(name)s in "
+                 "%(library)s") \
+              % {'name'   : self.name, 'library': self.library},
+              self.url)
 
   #
   # __findImportItem
