@@ -16,7 +16,7 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# Copyright 2001-2004 Free Software Foundation
+# Copyright 2001-2005 Free Software Foundation
 #
 # FILE:
 # xmlrpc/ServerAdapter.py
@@ -34,7 +34,7 @@
 #
 
 from gnue.common.rpc import server
-from gnue.common.apps import GDebug
+from gnue.common.apps import errors
 from gnue.common.rpc.drivers._helpers import ObjectLibrarian, DirectoryServer
 
 from BaseHTTPServer import BaseHTTPRequestHandler;
@@ -128,7 +128,7 @@ class ServerAdapter(DirectoryServer.DirectoryServer):
     if params.has_key('loglevel'):
       self._loglevel = params['loglevel']
     else:
-      self._loglevel = 1
+      self._loglevel = 0
 
     if params.has_key('httpbind'):
       self._httpbind = params['httpbind']
@@ -189,9 +189,14 @@ class ServerAdapter(DirectoryServer.DirectoryServer):
   #
   # Call the requested method
   #
+
+  # ---------------------------------------------------------------------------
+  # Call the requested method
+  # ---------------------------------------------------------------------------
+
   def call(self, method, params):
-    if self._loglevel>0:
-      print _("Dispatching: "), method, params 
+    if self._loglevel > 0:
+      print _("Dispatching: "), method, params
     
 
     ## Check if the Method is part of a service or a pointer to a
@@ -201,158 +206,162 @@ class ServerAdapter(DirectoryServer.DirectoryServer):
     ##                     params=""
     ## Call to an method: (of a service=one object)
     ##                     method="DonutPlace.Management.Restart"
+    try:
+      if method [0] == '[':
+        # call to an object
+        # 1. get the object from the objectlibrarian
+        # 2. check, if the object is supported by the gfd
 
-    if method[0]=='[':
-      # call to an object
-      # 1. get the object from the objectlibrarian
-      # 2. check, if the object is supported by the gfd
-      try:
-        i=string.index(method,']',1)
-        objhandle=method[1:i]
-        method=method[i+2:]
-      except ValueError:
-        tmsg = _("Wrong format of object handle ")+\
-              _("in method call %s") % method
-        raise AttributeError, tmsg
-      # TODO check in service dir, if obj is supported or not
-      o=ObjectLibrarian.retrieveObject(objhandle)
-      try:
-        server_method=getattr(o,method)
-        server_attribute=None
-      except AttributeError:
-        server_method=None
         try:
-          server_attribute=getattr(o,method[4:])
+          i = string.index (method, ']', 1)
+          objhandle = method [1:i]
+          method = method [i+2:]
+
+        except ValueError:
+          raise errors.ApplicationError, \
+              u_("Wrong format of object handle in method call %s") % method
+
+        # TODO check in service dir, if obj is supported or not
+        o = ObjectLibrarian.retrieveObject (objhandle)
+
+        try:
+          server_method = getattr (o, method)
+          server_attribute = None
+
         except AttributeError:
+          server_method = None
+
+          try:
+            server_attribute = getattr (o, method [4:])
+
+          except AttributeError:
+            if method != "_close":
+              raise errors.ApplicationError, \
+                  u_("Internal XMLRPC server error: method %s can be "
+                     "found in the directory (build out of a .grpc file), "
+                     "but the object doesn't contain this method/attribut. "
+                     "Please check you .grpc file for wrong return types.") \
+                   % method
           
-          if method!="_close":                      
-            msg=_("Internal XMLRPC server error: method %s can be ")% \
-                 method +\
-                 _("found in the directory (build out of a .grpc file),")+\
-                 _(" but the object doesn't contain this method/attribut.")+\
-                 _(" Please check you .grpc file for wrong return types.")
-            
-            raise AttributeError, msg
-        
-      if method!="_close":
-        direntry = self.getMethodDirEntry(o._type+"."+method)
-        signature=direntry['signature']
+        if method != "_close":
+          direntry  = self.getMethodDirEntry (o._type + "." + method)
+          signature = direntry ['signature']
+        else:
+          signature = ('string',)
+  
       else:
-        signature= ('string',)                
-
-    else:
-
-      # call to a service method or a helper call (get/set) for
-      # a service attribut
-      try:
-        direntry = self.getMethodDirEntry(method)
-        server_method = direntry['binding']
+        # call to a service method or a helper call (get/set) for
+        # a service attribut
+        direntry         = self.getMethodDirEntry (method)
+        server_method    = direntry ['binding']
         server_attribute = None
-        
+          
         # check if it is an real method (binding -> method)
         # or an get/set method for an attribut (binding-> attribut)
-        if (type(server_method)!=type(self.call)):
+
+        if (type (server_method) != type (self.call)):
           server_attribute = server_method
-          server_method=None
-          
-        signature=direntry['signature']
-
-        if (server_method==None) and (server_attribute==None):
-          tmsg = _("Server XML-RPC method %s  is not ")% method +\
-              _("bound to real method") 
-          raise AttributeError, tmsg
-      except KeyError:
-        tmsg = _("Server does not have XML-RPC ") +\
-              _("procedure %s") % method      
-        raise AttributeError, tmsg
-    try:
-      #
-      pass
-        # TODO:  Compare submitted attributs with signature
-    except KeyError:
-      tmsg = _("Server XML-RPC ") +\
-            _("procedure %s accepts just %s as attributs") % (method,attr)
-      raise AttributeError, tmsg
-    
-
-    # replace object handles in param with the real object
-    counter=0
-    while counter<len(params):
-      p=params[counter]
-      if type(p)==type(""):
-        if (len(p)==42) and (p[0]=="[") and (p[41]=="]"):
-          try:
-            p=p[1:41]
-            obj=ObjectLibrarian.retrieveObject(p)
-            newp=params[0:counter-1]+(obj,)+params[counter+1:]
-            params=newp
-          except:
-            pass
-      counter=counter+1;
-
-    # check if it is an real method (binding -> method)
-    # or an get/set method for an attribut (binding-> attribut)
-
-    __params = typeconv.rpc_to_python (params, server.InvalidParameter)
-
-    if (server_method!=None):
+          server_method    = None
             
-      # call method with params
+        signature = direntry ['signature']
+  
+        if (server_method is None) and (server_attribute is None):
+          raise errors.SystemError, \
+              u_("Server XML-RPC method '%s' is not bound to real method") \
+              % method
+  
       try:
-        result = server_method(*__params)
-      except:
-        return xmlrpclib.Fault (1, self._traceback (1))
-      else:
+        # TODO:  Compare submitted attributs with signature
+        pass
+
+      except KeyError:
+        raise errors.ApplicationError, \
+            u_("Server XML-RPC procedure %(method)s accepts just %(attr)s "
+               "as attributes") \
+            % {'method': method,
+               'attr'  : attr}
+      
+  
+      # replace object handles in param with the real object
+      counter = 0
+      while counter < len (params):
+        p = params [counter]
+        if type (p) == type (""):
+          if (len (p) == 42) and (p [0] == "[") and (p [41] == "]"):
+            try:
+              p = p [1:41]
+              obj = ObjectLibrarian.retrieveObject (p)
+              newp = params [0:counter-1] + (obj,) + params [counter+1:]
+              params = newp
+
+            except:
+              pass
+
+        counter += 1
+  
+      # check if it is an real method (binding -> method)
+      # or an get/set method for an attribut (binding-> attribut)
+  
+      __params = typeconv.rpc_to_python (params, server.InvalidParameter)
+  
+      if (server_method is not None):
+              
+        # call method with params
+        result = server_method (*__params)
         result = typeconv.python_to_rpc (result, server.InvalidParameter)
-
-    # check if it's the close object method
-    elif (method=="_close"):
-      
-      result=self.releaseDynamicObject(o)
-      
-    else:
-      
-      ## check wether its the set or the get method for the attribut
-      mparts=string.splitfields(method,'.')
-      mparts.reverse()
-      calltype=mparts[0]
-      calltype=calltype[:4]
-      GDebug.printMesg(7,'method %s has calling type %s' %\
-                       (method,calltype))
-      if calltype=='set_':
-        # setAttribut method
-        server_attribute=params[0]
-      elif calltype=='get_':
-        # getAttribut method
-        result=server_attribute
+  
+      # check if it's the close object method
+      elif (method == "_close"):
+        result = self.releaseDynamicObject (o)
+        
       else:
-        tmsg = _("Internal Server XML-RPC error: method type") +\
-            _("(get/set attribute) couldn't be detected (method %s)") \
-            % method       
-        raise AttributeError, tmsg
-    
+        
+        ## check wether it's the set or the get method for the attribute
+        mparts = string.splitfields (method, '.')
+        mparts.reverse ()
+        calltype = mparts [0]
+        calltype = calltype [:4]
+        gDebug (9, 'method %s has calling type %s' % (method, calltype))
 
-    # replace real object in param with an object handle
-    if type(result)==type(self):  ## both should be instances
-       ObjectLibrarian.archiveObject(result)
+        if calltype == 'set_':
+          # setAttribut method
+          server_attribute = params [0]
 
-       # get the type of the result
-       rtype=signature[0]
-       # delete the surrounding brackets < >
-       rtype=rtype[1:len(rtype)-1]
-       # store typeinfo in new object
-       result._type=rtype
-
-       result=ObjectLibrarian.getObjectReference(result)
-       self.registerDynamicObject(result,rtype)
-
-    # check for empty results (not allowed for XMLRPC)
-    if (result==None) or (result==[None]):
-      GDebug.printMesg(3,'Transform result None into 1')
-      result=1
+        elif calltype == 'get_':
+          # getAttribut method
+          result = server_attribute
+        else:
+          raise errors.SystemError, \
+              u_("Internal Server XML-RPC error: method type (get/set "
+                 "attribute) couldn't be detected (method %s)") % method
       
-    return result
+  
+      # replace real object in param with an object handle
+      if type (result) == type (self):  ## both should be instances
+         ObjectLibrarian.archiveObject (result)
+  
+         # get the type of the result
+         rtype = signature [0]
+         # delete the surrounding brackets < >
+         rtype = rtype [1:len(rtype)-1]
+         # store typeinfo in new object
+         result._type = rtype
+  
+         result = ObjectLibrarian.getObjectReference (result)
+         self.registerDynamicObject (result, rtype)
+  
+      # check for empty results (not allowed for XMLRPC)
+      if (result is None) or (result == [None]):
+        gDebug (9, 'Transform result None into 1')
+        result = 1
 
+    except:
+      stack  = string.join (errors.getException (1), u'\x91')
+      result = xmlrpclib.Fault (1, stack)
+
+    return result
+  
+  
   def registerDynamicObject(self,objhandle,type):
     #
     #  add new methods to the xmlrpc server
@@ -361,9 +370,8 @@ class ServerAdapter(DirectoryServer.DirectoryServer):
     for i in self.directory.keys():
       if i[0:tl]==type:
         method="["+objhandle+"]"+i[tl:]
-        GDebug.printMesg(1,'Method %s registered to py-xmlrpc ' \
-                         % method +\
-                         ' internal directory.')
+        gDebug (9, 'Method %s registered to py-xmlrpc internal directory.' \
+                   % method)
  #       self.server.addMethods({method:None})
      
     # add a method to close this object
@@ -379,7 +387,7 @@ class ServerAdapter(DirectoryServer.DirectoryServer):
     for i in self.directory.keys():
       if i[0:tl]==type:
         method="["+objhandle+"]"+i[tl:]
-        GDebug.printMesg(1,'Method %s is deleted from py-xmlrpc ' \
+        gDebug (9, 'Method %s is deleted from py-xmlrpc ' \
                          % method +\
                          ' internal directory.')
 #        self.server.removeMethods({method:None})
@@ -403,6 +411,10 @@ class ServerAdapter(DirectoryServer.DirectoryServer):
 
 class RequestHandler(BaseHTTPRequestHandler):
 
+  def log_message (self, format, *args):
+    gDebug (9, "%s - - [%s] %s" % \
+        (self.address_string(), self.log_date_time_string(), format%args))
+
   # override basic handle to test allowed host condition
   #
   def handle(self):
@@ -446,9 +458,8 @@ class RequestHandler(BaseHTTPRequestHandler):
         
     except:
       # internal error, report as HTTP server error
-      GDebug.printMesg(1,
-                       'Unexpected Exception in XML-RPC code: %s:%s' % \
-                       (sys.exc_type, sys.exc_value))
+      gDebug (1, 'Unexpected Exception in XML-RPC code: %s:%s' % \
+                 (sys.exc_type, sys.exc_value))
       # TODO: try to be a bit more informative (on client side)
       self.send_response(500)
       self.end_headers()
@@ -548,7 +559,7 @@ class _ObjectByReferenceHandler:
         del args['obj_handle']
 
       except KeyError:
-        tmsg = _('Object handle not returned') 
+        tmsg = _('Object handle not returned')
         raise StandardError, tmsg   # TODO
 
     try:
@@ -557,7 +568,7 @@ class _ObjectByReferenceHandler:
     except KeyError:
 
       # Attempt to use an invalid objecthandle
-      tmsg = _('Invalid object handle') 
+      tmsg = _('Invalid object handle')
       raise StandardError, tmsg # TODO
 
 
